"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KnapsackWebComponentRenderer = void 0;
const app_1 = require("@knapsack/app");
const fs_extra_1 = require("fs-extra");
const types_1 = require("@knapsack/types");
const path_1 = require("path");
const utils_1 = require("./utils");
const pkg = JSON.parse((0, fs_extra_1.readFileSync)((0, path_1.join)(__dirname, '../package.json'), 'utf-8'));
app_1.log.setupUpdateNotifier(pkg);
/* eslint-disable class-methods-use-this */
class KnapsackWebComponentRenderer extends app_1.KnapsackRendererBase {
    constructor() {
        super({
            id: 'web-components',
            language: 'html',
            extension: '.html',
        });
        this.getMeta = () => ({
            id: this.id,
            title: 'Web Components',
            aliasUse: 'required',
            aliasTitle: 'Custom Element Name',
            aliasDescription: 'The custom element tag name (ex. my-card)',
            enableDataDemos: true,
            enableTemplateDemos: true,
            hasSlotsSupport: true,
            version: pkg.version,
            hasInferSpecSupport: false,
            hasTemplateSuggestionsSupport: false,
        });
        this.getTemplateMeta = async ({ pattern, template, }) => {
            var _a, _b;
            const files = [];
            const htmlCustomData = {
                version: 1.1,
                tags: [],
            };
            if ((_a = template === null || template === void 0 ? void 0 : template.spec) === null || _a === void 0 ? void 0 : _a.props) {
                const typeDefs = await KnapsackWebComponentRenderer.convertSchemaToTypeScriptDefs({
                    schema: template.spec.props,
                    title: pattern.id,
                    patternId: pattern.id,
                    templateId: template.id,
                });
                const attributes = [];
                Object.entries(template.spec.props.properties).forEach(([name, meta]) => {
                    var _a, _b, _c;
                    const descriptions = [];
                    const attr = {
                        name,
                    };
                    const isRequired = (_c = (_b = (_a = template.spec.props) === null || _a === void 0 ? void 0 : _a.required) === null || _b === void 0 ? void 0 : _b.includes(name)) !== null && _c !== void 0 ? _c : false;
                    if (isRequired) {
                        descriptions.push('(required)');
                    }
                    if ((0, types_1.isOptionsProp)(meta)) {
                        attr.values = meta.enum.map((option) => ({
                            name: option,
                        }));
                    }
                    else if ((0, types_1.isBooleanProp)(meta)) {
                        descriptions.push('boolean');
                    }
                    if (meta.description)
                        descriptions.push(meta.description);
                    attr.description = descriptions.join(' ');
                    attributes.push(attr);
                });
                htmlCustomData.tags.push({
                    name: template.alias,
                    description: (_b = pattern.description) !== null && _b !== void 0 ? _b : '',
                    references: [
                        {
                            name: 'Knapsack Docs',
                            // @todo pull in base url
                            url: `http://localhost:3999/pattern/${pattern.id}/${template.id}`,
                        },
                    ],
                    attributes,
                });
                files.push({
                    contents: JSON.stringify(htmlCustomData, null, '  '),
                    encoding: 'utf8',
                    path: `${pattern.id}.${template.id}.html-data.json`,
                });
                files.push({
                    contents: typeDefs,
                    encoding: 'utf8',
                    path: `${pattern.id}.${template.id}.spec.d.ts`,
                });
                files.push({
                    contents: JSON.stringify(template.spec.props, null, '  '),
                    encoding: 'utf8',
                    path: `${pattern.id}.${template.id}.spec.json`,
                });
            }
            return files;
        };
        this.alterTemplateMetaFiles = async ({ files }) => {
            const newFiles = [];
            const htmlCustomData = {
                version: 1.1,
                tags: [],
            };
            files.forEach((file) => {
                if (!file.path.endsWith('html-data.json')) {
                    newFiles.push(file);
                }
                else {
                    const data = JSON.parse(file.contents);
                    data.tags.forEach((tag) => htmlCustomData.tags.push(tag));
                }
            });
            newFiles.push({
                contents: JSON.stringify(htmlCustomData, null, '  '),
                encoding: 'utf8',
                path: `knapsack.html-data.json`,
            });
            return newFiles;
        };
    }
    async render(opt) {
        const usage = await this.getUsage(opt);
        return {
            ok: true,
            html: app_1.KnapsackRendererBase.formatCode({
                code: usage,
                language: 'html',
            }),
        };
    }
    async getUsage(opt) {
        const { pattern, template, demo, 
        /**
         * The Patterns class
         */
        patternManifest, } = opt;
        if (!demo) {
            return (0, utils_1.getUsage)({
                templateName: template.alias,
            });
        }
        switch (demo.type) {
            case 'template': {
                const { templateInfo } = demo;
                const templateDemoPath = patternManifest.getTemplateDemoAbsolutePath({
                    patternId: pattern.id,
                    templateId: template.id,
                    demoId: demo.id,
                });
                return (0, fs_extra_1.readFile)(templateDemoPath, 'utf8');
            }
            case 'data': {
                const { data: { props, slots }, } = demo;
                if (!slots) {
                    return (0, utils_1.getUsage)({
                        templateName: template.alias,
                        props,
                    });
                }
                const children = [];
                await Promise.all(Object.keys(slots).map(async (slotName) => {
                    const slottedTemplates = slots[slotName];
                    await Promise.all(slottedTemplates.map(async (slottedTemplate) => {
                        if (app_1.KnapsackRendererBase.isSlottedText(slottedTemplate)) {
                            return slottedTemplate.text;
                        }
                        if (app_1.KnapsackRendererBase.isSlottedTemplateReference(slottedTemplate)) {
                            throw new Error(`Cannot handle slotted template references in ${JSON.stringify({
                                patternId: pattern.id,
                                templateId: template.id,
                                demoId: demo.id,
                                slottedTemplate,
                            })}`);
                        }
                        if (!slottedTemplate.patternId ||
                            !slottedTemplate.templateId ||
                            !slottedTemplate.demoId) {
                            return '';
                        }
                        const thisPattern = patternManifest.getPattern(slottedTemplate.patternId);
                        const thisTemplate = thisPattern.templates.find((t) => t.id === slottedTemplate.templateId);
                        if (!thisTemplate) {
                            throw new Error(`Cannot find template id "${slottedTemplate.templateId}" in "${slottedTemplate.patternId}"`);
                        }
                        const usage = await this.getUsage({
                            pattern: thisPattern,
                            template: thisTemplate,
                            patternManifest,
                            demo: slottedTemplate.demo ||
                                this.patterns.demosById[slottedTemplate.demoId],
                        });
                        return usage;
                    })).then((usages) => {
                        if (slotName === 'default') {
                            children.push(usages.join('\n'));
                        }
                        else {
                            children.push(`<div slot="${slotName}">${usages.join('\n')}</div>`);
                        }
                    });
                }));
                const usage = await (0, utils_1.getUsage)({
                    templateName: template.alias,
                    props,
                    children: children.join('\n'),
                });
                return usage;
            }
            default:
                throw new Error(`Must pass in a demo.type`);
        }
    }
}
exports.KnapsackWebComponentRenderer = KnapsackWebComponentRenderer;
//# sourceMappingURL=renderer-web-components.js.map